unit Demo_Au1;

interface

uses
{$IFDEF WIN32}
  Windows,
{$ELSE}
  WinTypes, WinProcs,
{$ENDIF}
  SysUtils, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Audio, StdCtrls, Spin, Buttons, Gauges, ExtCtrls;

type

  TMainForm = class(TForm)
    Audio1: TAudio;
    Memo1: TMemo;
    Query: TButton;
    StartRec: TButton;
    StopRec: TButton;
    Status: TLabel;
    StartPlay: TButton;
    StopPlay: TButton;
    PlayFile: TButton;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    RecFile: TButton;
    cbLink: TCheckBox;
    PauseRec: TSpeedButton;
    LeftScroll: TScrollBar;
    RightScroll: TScrollBar;
    Label1: TLabel;
    TrigLvl: TScrollBar;
    Label2: TLabel;
    Label3: TLabel;
    SourceSel: TComboBox;
    cbMute: TCheckBox;
    cbLoop: TCheckBox;
    DestSel: TComboBox;
    SaveToFile: TButton;
    Button1: TButton;
    Gauge1: TGauge;
    SpeedButton1: TSpeedButton;
    procedure ButtonStatus(StatusText:string);
    procedure QueryClick(Sender: TObject);
    procedure StartRecClick(Sender: TObject);
    procedure StopRecClick(Sender: TObject);
    procedure Audio1BufferPlayed(Sender: TObject);
    procedure Audio1Record(Sender: TObject; LP,RP: Pointer; BufferSize: Word);
    procedure StartPlayClick(Sender: TObject);
    procedure StopPlayClick(Sender: TObject);
    procedure PlayFileClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure RecFileClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cbLinkClick(Sender: TObject);
    procedure PauseRecClick(Sender: TObject);
    procedure TrigLvlChange(Sender: TObject);
    procedure LeftScrollChange(Sender: TObject);
    procedure RightScrollChange(Sender: TObject);
    procedure SourceSelChange(Sender: TObject);
    procedure cbMuteClick(Sender: TObject);
    procedure Audio1Played(Sender: TObject);
    procedure NewVol(LeftCtrl:boolean);
    procedure DestSelChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SaveToFileClick(Sender: TObject);
    procedure cbLoopClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Audio1MixerChange(Sender: TObject; Destination,
      Source: Word);
    procedure SpeedButton1Click(Sender: TObject);
  public
    LeftStream,RightStream:TMemoryStream;
    NoOfLoops : byte;
    audiorecording, recordandplay:boolean;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

procedure TMainForm.ButtonStatus(StatusText:string);
begin
  Status.Caption := StatusText;
  Status.Font.Color := clBlack;
    StartRec.Enabled := False;
    StopRec.Enabled := False;
    PauseRec.Enabled := False;
    StartPlay.Enabled := False;
    StopPlay.Enabled := False;
    PlayFile.Enabled := False;
    RecFile.Enabled := False;
    SaveToFile.Enabled:=False;
    Query.Enabled:=False;
  if StatusText='Idle' then begin
    StartRec.Enabled := True;
    StartPlay.Enabled := True;
    PlayFile.Enabled := True;
    RecFile.Enabled := True;
    SaveToFile.Enabled:=True;
    Query.Enabled:=True;
  end;
  if StatusText='Recording' then begin
    StopRec.Enabled := True;
    PauseRec.Enabled := True;
    PauseRec.Down:=False;
  end;
  if StatusText='Playing' then begin
    StopPlay.Enabled := True;
  end;
end;

procedure TMainForm.QueryClick(Sender: TObject);
var
  Product, Formats: string;
begin
  Audio1.Query(Product, Formats);
  Memo1.Lines.Add(Product);
  Memo1.Lines.Add(Formats);
  Audio1.Mixer.Query(Product, Formats);
  Memo1.Lines.Add(Product);
  Memo1.Lines.Add(Formats);
end;

procedure TMainForm.StartRecClick(Sender: TObject);
begin
  Audio1.Recorder.Triggered:=false;
  if not(Audio1.Recorder.Start) then Memo1.Lines.Add(Audio1.ErrorMessage);
  if LeftStream <> nil then begin
    LeftStream.Free; RightStream.Free;
    LeftStream := nil; RightStream:=nil;
  end;
  try
    LeftStream :=TMemoryStream.Create;
    RightStream:=TMemoryStream.Create;
  except
    LeftStream:=nil; RightStream:=nil;
    if not(Audio1.Recorder.Stop) then Memo1.Lines.Add(Audio1.ErrorMessage);
    exit;
  end;
  audiorecording:=true;
  ButtonStatus('Recording');
end;

procedure TMainForm.Audio1Record(Sender: TObject; LP,RP: Pointer;
  BufferSize: Word);
var LMeter, RMeter : longint;
begin
  if BufferSize>0 then begin
    if recordandplay then begin
      LeftStream.Position:=0;
      if RP<>nil then RightStream.Position:=0;
    end;
    LeftStream.Write(LP^,BufferSize);
    if RP<>nil then RightStream.Write(RP^,BufferSize);
    if Status.Font.Color = clYellow then Status.Font.Color := clGreen
    else Status.Font.Color := clYellow;
{$IFDEF WIN32}
    Audio1.Mixer.GetMeter(1,0,LMeter,RMeter);
{$ENDIF}
    Gauge1.Progress:=LMeter;
    if recordandplay then Audio1.Player.Play(LeftStream,RightStream,0);
  end else begin
    if not(Audio1.Recorder.Stop) then begin
      Memo1.Lines.Add(Audio1.ErrorMessage);
    end;
    ButtonStatus('Idle');
  end;;    
end;

procedure TMainForm.StopRecClick(Sender: TObject);
begin
  if not(Audio1.Recorder.Stop) then begin
    Memo1.Lines.Add(Audio1.ErrorMessage);
  end;
  audiorecording:=false;
  ButtonStatus('Idle');
end;

procedure TMainForm.StartPlayClick(Sender: TObject);
begin
  if (LeftStream<>nil) and (LeftStream.Size>0) then begin
    LeftStream.Position:=0;
    if (RightStream<>nil) and (RightStream.Size>0) then RightStream.Position:=0
    else begin
      RightStream.Free;
      RightStream:=nil;
    end;
    ButtonStatus('Playing');
    Status.Font.Color := clRed;
    Audio1.Player.Play(LeftStream,RightStream,NoOfLoops);
  end;
end;

procedure TMainForm.Audio1BufferPlayed(Sender: TObject);
begin
  if Status.Font.Color = clRed then Status.Font.Color := clBlue
  else Status.Font.Color := clRed;
end;

procedure TMainForm.Audio1Played(Sender: TObject);
begin
  ButtonStatus('Idle');
end;

procedure TMainForm.StopPlayClick(Sender: TObject);
begin
  Audio1.Player.Stop;
  ButtonStatus('Idle');
end;

procedure TMainForm.PlayFileClick(Sender: TObject);
begin
  if OpenDialog1.Execute then begin
    Memo1.Lines.Add('Playing file : ' + ExtractFileName(OpenDialog1.FileName));
    if not(Audio1.Player.PlayFile(OpenDialog1.FileName,NoOfLoops)) then
      Memo1.Lines.Add(Audio1.ErrorMessage)
    else ButtonStatus('Playing');
  end;
end;

procedure TMainForm.FormActivate(Sender: TObject);
var
  LVol, RVol: word;
  Mute : boolean;
  CtrlType : byte;
begin
{$IFDEF WIN32}
  Audio1.Mixer.GetControl(word(DestSel.ItemIndex),word(SourceSel.ItemIndex),LVol,RVol,Mute,CtrlType);
  cbMute.Enabled:=((CtrlType and 128)=0);
  cbMute.Checked:=Mute;
  RightScroll.Visible:=((CtrlType and 64)=0);
{$ELSE}
  Audio1.Player.GetVolume(LVol, RVol);
{$ENDIF}
  LeftScroll.Position:=round(100/65535*(65535-LVol));
  RightScroll.Position:=round(100/65535*(65535-RVol));
end;

procedure TMainForm.RecFileClick(Sender: TObject);
begin
  if SaveDialog1.Execute then begin
    Audio1.Recorder.RecordToFile(SaveDialog1.FileName, nil,nil);
    Memo1.Lines.Add('Recording to file : ' + ExtractFileName(SaveDialog1.FileName));
    StartRecClick(Self);    
  end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
{$IFDEF WIN32}
  DestSel.Items:=Audio1.Mixer.GetDestinations;   
  DestSel.ItemIndex:=0;
  SourceSel.Items:=Audio1.Mixer.GetSources(DestSel.ItemIndex);
  SourceSel.ItemIndex:=0;
{$ELSE}
  SourceSel.Items.Clear;
  SourceSel.Items.Add('Wave');
  SourceSel.ItemIndex:=0;
  cbMute.Enabled:=false;
{$ENDIF}
  TrigLvlChange(Self);
  LeftStream:=nil; RightStream:=nil;
  NoOfLoops:=0;
  recordandplay:=false;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  if LeftStream<>nil then LeftStream.Free;
  if RightStream<>nil then RightStream.Free;
end;

procedure TMainForm.cbLinkClick(Sender: TObject);
begin
  RightScrollChange(Sender);
end;

procedure TMainForm.PauseRecClick(Sender: TObject);
begin
  if PauseRec.Down then Audio1.Recorder.Pause
  else Audio1.Recorder.Restart;
end;

procedure TMainForm.TrigLvlChange(Sender: TObject);
begin
  if Audio1.Recorder.BitsPerSample=_8 then
    Audio1.Recorder.TrigLevel:=round(127*((100-TrigLvl.Position)/100))+127
  else
    Audio1.Recorder.TrigLevel:=round(32767*((100-TrigLvl.Position)/100));
end;

procedure TMainForm.NewVol(LeftCtrl:boolean);
var LVol, RVol : Word;
begin
  LVol:=round(65535/100*(100-LeftScroll.Position));
  RVol:=round(65535/100*(100-RightScroll.Position));
  if cbLink.Checked then begin
    if LeftCtrl then RVol:=LVol
    else LVol:=RVol;
  end;
{$IFDEF WIN32}
 if not(Audio1.Mixer.SetControl(word(DestSel.ItemIndex),word(SourceSel.ItemIndex),LVol,RVol,cbMute.Checked)) then begin
   Memo1.Lines.Add(Audio1.ErrorMessage);
 end;
{$ELSE}
  Audio1.Player.SetVolume(LVol,RVol);
  FormActivate(Self);    
{$ENDIF}
end;

procedure TMainForm.LeftScrollChange(Sender: TObject);
begin
  NewVol(true);
end;

procedure TMainForm.RightScrollChange(Sender: TObject);
begin
  NewVol(false);
end;

procedure TMainForm.SourceSelChange(Sender: TObject);
begin
  FormActivate(Self);
end;

procedure TMainForm.cbMuteClick(Sender: TObject);
begin
  NewVol(true);
end;

procedure TMainForm.DestSelChange(Sender: TObject);
begin
{$IFDEF WIN32}
  SourceSel.Items:=Audio1.Mixer.GetSources(DestSel.ItemIndex);
{$ENDIF}
  SourceSel.ItemIndex:=0;
  FormActivate(Self);
end;

procedure TMainForm.SaveToFileClick(Sender: TObject);
begin
  if SaveDialog1.Execute then begin
    Audio1.Recorder.RecordToFile(SaveDialog1.FileName, LeftStream,RightStream);
    Memo1.Lines.Add('Save sound : ' + ExtractFileName(SaveDialog1.FileName));
  end;
end;

procedure TMainForm.cbLoopClick(Sender: TObject);
begin
  if cbLoop.Checked then NoOfLoops:=1
  else NoOfLoops:=0;
end;

procedure TMainForm.Button1Click(Sender: TObject);
begin
{ Testing simultaneous playing -> recording.
  Requires that the audio output is connected to the audio input }
  if OpenDialog1.Execute then begin
    Memo1.Lines.Add('Playing file : ' + ExtractFileName(OpenDialog1.FileName));
    if not(Audio1.Player.PlayFile(OpenDialog1.FileName,NoOfLoops)) then
      Memo1.Lines.Add(Audio1.ErrorMessage);
    Audio1.Recorder.Triggered:=false;
    if not(Audio1.Recorder.Start) then Memo1.Lines.Add(Audio1.ErrorMessage);
    if LeftStream <> nil then begin
      LeftStream.Free; RightStream.Free;
      LeftStream := nil; RightStream:=nil;
    end;
    try
      LeftStream :=TMemoryStream.Create;
      RightStream:=TMemoryStream.Create;
    except
      LeftStream:=nil; RightStream:=nil;
      if not(Audio1.Recorder.Stop) then Memo1.Lines.Add(Audio1.ErrorMessage);
      exit;
    end;
    ButtonStatus('Recording');
  end;
end;

procedure TMainForm.Audio1MixerChange(Sender: TObject; Destination,
  Source: Word);
begin
  if (Destination=DestSel.ItemIndex) and (Source=SourceSel.ItemIndex) then
    FormActivate(Self);
end;

procedure TMainForm.SpeedButton1Click(Sender: TObject);
begin
{ Testing simultaneous recording -> playing }
  if SpeedButton1.Down then begin
    recordandplay:=true;
    if not(Audio1.Recorder.Start) then Memo1.Lines.Add(Audio1.ErrorMessage);
    if LeftStream <> nil then begin
      LeftStream.Free; RightStream.Free;
      LeftStream := nil; RightStream:=nil;
    end;
    try
      LeftStream :=TMemoryStream.Create;
      RightStream:=TMemoryStream.Create;
    except
      LeftStream:=nil; RightStream:=nil;
      if not(Audio1.Recorder.Stop) then Memo1.Lines.Add(Audio1.ErrorMessage);
      exit;
    end;
    ButtonStatus('Recording');
  end else begin
    recordandplay:=false;
    if not(Audio1.Recorder.Stop) then begin
      Memo1.Lines.Add(Audio1.ErrorMessage);
    end;
    Audio1.Player.Stop;
    audiorecording:=false;
    ButtonStatus('Idle');
  end;
end;

end.

